@@ -31,11 +31,12 @@
  * Parse a command from a string.
  */
 
-int	cmd_string_getc(const char *, size_t *);
-void	cmd_string_ungetc(size_t *);
-char   *cmd_string_string(const char *, size_t *, char, int);
-char   *cmd_string_variable(const char *, size_t *);
-char   *cmd_string_expand_tilde(const char *, size_t *);
+int	 cmd_string_getc(const char *, size_t *);
+void	 cmd_string_ungetc(size_t *);
+void	 cmd_string_copy(char **, char *, size_t *);
+char	*cmd_string_string(const char *, size_t *, char, int);
+char	*cmd_string_variable(const char *, size_t *);
+char	*cmd_string_expand_tilde(const char *, size_t *);
 
 int
 cmd_string_getc(const char *s, size_t *p)
@@ -58,7 +59,8 @@ cmd_string_ungetc(size_t *p)
  * string, or NULL for empty command.
  */
 int
-cmd_string_parse(const char *s, struct cmd_list **cmdlist, char **cause)
+cmd_string_parse(const char *s, struct cmd_list **cmdlist, const char *file,
+    u_int line, char **cause)
 {
 	size_t		p;
 	int		ch, i, argc, rval;
@@ -84,26 +86,17 @@ cmd_string_parse(const char *s, struct cmd_list **cmdlist, char **cause)
 		case '\'':
 			if ((t = cmd_string_string(s, &p, '\'', 0)) == NULL)
 				goto error;
-			buf = xrealloc(buf, 1, len + strlen(t) + 1);
-			strlcpy(buf + len, t, strlen(t) + 1);
-			len += strlen(t);
-			free(t);
+			cmd_string_copy(&buf, t, &len);
 			break;
 		case '"':
 			if ((t = cmd_string_string(s, &p, '"', 1)) == NULL)
 				goto error;
-			buf = xrealloc(buf, 1, len + strlen(t) + 1);
-			strlcpy(buf + len, t, strlen(t) + 1);
-			len += strlen(t);
-			free(t);
+			cmd_string_copy(&buf, t, &len);
 			break;
 		case '$':
 			if ((t = cmd_string_variable(s, &p)) == NULL)
 				goto error;
-			buf = xrealloc(buf, 1, len + strlen(t) + 1);
-			strlcpy(buf + len, t, strlen(t) + 1);
-			len += strlen(t);
-			free(t);
+			cmd_string_copy(&buf, t, &len);
 			break;
 		case '#':
 			/* Comment: discard rest of line. */
@@ -139,7 +132,7 @@ cmd_string_parse(const char *s, struct cmd_list **cmdlist, char **cause)
 			if (argc == 0)
 				goto out;
 
-			*cmdlist = cmd_list_parse(argc, argv, cause);
+			*cmdlist = cmd_list_parse(argc, argv, file, line, cause);
 			if (*cmdlist == NULL)
 				goto out;
 
@@ -147,12 +140,10 @@ cmd_string_parse(const char *s, struct cmd_list **cmdlist, char **cause)
 			goto out;
 		case '~':
 			if (buf == NULL) {
-				if ((t = cmd_string_expand_tilde(s, &p)) == NULL)
+				t = cmd_string_expand_tilde(s, &p);
+				if (t == NULL)
 					goto error;
-				buf = xrealloc(buf, 1, len + strlen(t) + 1);
-				strlcpy(buf + len, t, strlen(t) + 1);
-				len += strlen(t);
-				free(t);
+				cmd_string_copy(&buf, t, &len);
 				break;
 			}
 			/* FALLTHROUGH */
@@ -181,6 +172,20 @@ out:
 	return (rval);
 }
 
+void
+cmd_string_copy(char **dst, char *src, size_t *len)
+{
+	size_t srclen;
+
+	srclen = strlen(src);
+
+	*dst = xrealloc(*dst, 1, *len + srclen + 1);
+	strlcpy(*dst + *len, src, srclen + 1);
+
+	*len += srclen;
+	free(src);
+}
+
 char *
 cmd_string_string(const char *s, size_t *p, char endch, int esc)
 {
@@ -220,10 +225,7 @@ cmd_string_string(const char *s, size_t *p, char endch, int esc)
 				break;
 			if ((t = cmd_string_variable(s, p)) == NULL)
 				goto error;
-			buf = xrealloc(buf, 1, len + strlen(t) + 1);
-			strlcpy(buf + len, t, strlen(t) + 1);
-			len += strlen(t);
-			free(t);
+			cmd_string_copy(&buf, t, &len);
 			continue;
 		}
 
