@@ -46,8 +46,12 @@ control_notify_input(struct client *c, struct window_pane *wp,
 	if (winlink_find_by_window(&c->session->windows, wp->window) != NULL) {
 		message = evbuffer_new();
 		evbuffer_add_printf(message, "%%output %%%u ", wp->id);
-		for (i = 0; i < len; i++)
-			evbuffer_add_printf(message, "%02hhx", buf[i]);
+		for (i = 0; i < len; i++) {
+			if (buf[i] < ' ' || buf[i] == '\\')
+			    evbuffer_add_printf(message, "\\%03o", buf[i]);
+			else
+			    evbuffer_add_printf(message, "%c", buf[i]);
+		}
 		control_write_buffer(c, message);
 		evbuffer_free(message);
 	}
@@ -95,19 +99,14 @@ void
 control_notify_window_unlinked(unused struct session *s, struct window *w)
 {
 	struct client	*c;
-	struct session	*cs;
 	u_int		 i;
 
 	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
 		c = ARRAY_ITEM(&clients, i);
 		if (!CONTROL_SHOULD_NOTIFY_CLIENT(c) || c->session == NULL)
 			continue;
-		cs = c->session;
 
-		if (winlink_find_by_window_id(&cs->windows, w->id) != NULL)
-			control_write(c, "%%window-close %u", w->id);
-		else
-			control_write(c, "%%unlinked-window-close %u", w->id);
+		control_write(c, "%%window-close @%u", w->id);
 	}
 }
 
@@ -125,9 +124,9 @@ control_notify_window_linked(unused struct session *s, struct window *w)
 		cs = c->session;
 
 		if (winlink_find_by_window_id(&cs->windows, w->id) != NULL)
-			control_write(c, "%%window-add %u", w->id);
+			control_write(c, "%%window-add @%u", w->id);
 		else
-			control_write(c, "%%unlinked-window-add %u", w->id);
+			control_write(c, "%%unlinked-window-add @%u", w->id);
 	}
 }
 
@@ -135,22 +134,14 @@ void
 control_notify_window_renamed(struct window *w)
 {
 	struct client	*c;
-	struct session	*s;
 	u_int		 i;
 
 	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
 		c = ARRAY_ITEM(&clients, i);
 		if (!CONTROL_SHOULD_NOTIFY_CLIENT(c) || c->session == NULL)
 			continue;
-		s = c->session;
 
-		if (winlink_find_by_window_id(&s->windows, w->id) != NULL) {
-			control_write(c, "%%window-renamed %u %s",
-			    w->id, w->name);
-		} else {
-			control_write(c, "%%unlinked-window-renamed %u %s",
-			    w->id, w->name);
-		}
+		control_write(c, "%%window-renamed @%u %s", w->id, w->name);
 	}
 }
 
@@ -163,7 +154,7 @@ control_notify_attached_session_changed(struct client *c)
 		return;
 	s = c->session;
 
-	control_write(c, "%%session-changed %d %s", s->idx, s->name);
+	control_write(c, "%%session-changed $%u %s", s->id, s->name);
 }
 
 void
@@ -174,10 +165,10 @@ control_notify_session_renamed(struct session *s)
 
 	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
 		c = ARRAY_ITEM(&clients, i);
-		if (!CONTROL_SHOULD_NOTIFY_CLIENT(c) || c->session != s)
+		if (!CONTROL_SHOULD_NOTIFY_CLIENT(c))
 			continue;
 
-		control_write(c, "%%session-renamed %s", s->name);
+		control_write(c, "%%session-renamed $%u %s", s->id, s->name);
 	}
 }
 
@@ -189,7 +180,7 @@ control_notify_session_created(unused struct session *s)
 
 	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
 		c = ARRAY_ITEM(&clients, i);
-		if (!CONTROL_SHOULD_NOTIFY_CLIENT(c) || c->session == NULL)
+		if (!CONTROL_SHOULD_NOTIFY_CLIENT(c))
 			continue;
 
 		control_write(c, "%%sessions-changed");
@@ -204,7 +195,7 @@ control_notify_session_close(unused struct session *s)
 
 	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
 		c = ARRAY_ITEM(&clients, i);
-		if (!CONTROL_SHOULD_NOTIFY_CLIENT(c) || c->session == NULL)
+		if (!CONTROL_SHOULD_NOTIFY_CLIENT(c))
 			continue;
 
 		control_write(c, "%%sessions-changed");
