@@ -48,7 +48,7 @@ char		 socket_path[MAXPATHLEN];
 int		 login_shell;
 char		*environ_path;
 pid_t		 environ_pid = -1;
-int		 environ_idx = -1;
+int		 environ_session_id = -1;
 
 __dead void	 usage(void);
 void	 	 parseenvironment(void);
@@ -147,22 +147,22 @@ parseenvironment(void)
 {
 	char	*env, path[256];
 	long	 pid;
-	int	 idx;
+	int	 id;
 
 	if ((env = getenv("TMUX")) == NULL)
 		return;
 
-	if (sscanf(env, "%255[^,],%ld,%d", path, &pid, &idx) != 3)
+	if (sscanf(env, "%255[^,],%ld,%d", path, &pid, &id) != 3)
 		return;
 	environ_path = xstrdup(path);
 	environ_pid = pid;
-	environ_idx = idx;
+	environ_session_id = id;
 }
 
 char *
 makesocketpath(const char *label)
 {
-	char		base[MAXPATHLEN], *path, *s;
+	char		base[MAXPATHLEN], realbase[MAXPATHLEN], *path, *s;
 	struct stat	sb;
 	u_int		uid;
 
@@ -186,7 +186,10 @@ makesocketpath(const char *label)
 		return (NULL);
 	}
 
-	xasprintf(&path, "%s/%s", base, label);
+	if (realpath(base, realbase) == NULL)
+		strlcpy(realbase, base, sizeof realbase);
+
+	xasprintf(&path, "%s/%s", realbase, label);
 	return (path);
 }
 
@@ -390,8 +393,7 @@ main(int argc, char **argv)
 		}
 	}
 	free(label);
-	if (realpath(path, socket_path) == NULL)
-		strlcpy(socket_path, path, sizeof socket_path);
+	strlcpy(socket_path, path, sizeof socket_path);
 	free(path);
 
 #ifdef HAVE_SETPROCTITLE
