@@ -283,7 +283,6 @@ window_create1(u_int sx, u_int sy)
 
 	w->lastlayout = -1;
 	w->layout_root = NULL;
-	TAILQ_INIT(&w->layout_list);
 
 	w->sx = sx;
 	w->sy = sy;
@@ -308,24 +307,27 @@ window_create1(u_int sx, u_int sy)
 struct window *
 window_create(const char *name, const char *cmd, const char *shell,
     const char *cwd, struct environ *env, struct termios *tio,
-    u_int sx, u_int sy, u_int hlimit,char **cause)
+    u_int sx, u_int sy, u_int hlimit, char **cause)
 {
 	struct window		*w;
 	struct window_pane	*wp;
 
 	w = window_create1(sx, sy);
 	wp = window_add_pane(w, hlimit);
-	layout_init(w);
+	layout_init(w, wp);
+
 	if (window_pane_spawn(wp, cmd, shell, cwd, env, tio, cause) != 0) {
 		window_destroy(w);
 		return (NULL);
 	}
+
 	w->active = TAILQ_FIRST(&w->panes);
 	if (name != NULL) {
 		w->name = xstrdup(name);
 		options_set_number(&w->options, "automatic-rename", 0);
 	} else
 		w->name = default_window_name(w);
+
 	return (w);
 }
 
@@ -334,6 +336,8 @@ window_destroy(struct window *w)
 {
 	u_int	i;
 
+	window_unzoom(w);
+
 	if (window_index(w, &i) != 0)
 		fatalx("index not found");
 	ARRAY_SET(&windows, i, NULL);
@@ -456,6 +460,57 @@ window_find_string(struct window *w, const char *s)
 	return (window_get_active_at(w, x, y));
 }
 
+int
+window_zoom(struct window_pane *wp)
+{
+	struct window		*w = wp->window;
+	struct window_pane	*wp1;
+
+	if (w->flags & WINDOW_ZOOMED)
+		return (-1);
+
+	if (!window_pane_visible(wp))
+		return (-1);
+
+	if (window_count_panes(w) == 1)
+		return (-1);
+
+	if (w->active != wp)
+		window_set_active_pane(w, wp);
+
+	TAILQ_FOREACH(wp1, &w->panes, entry) {
+		wp1->saved_layout_cell = wp1->layout_cell;
+		wp1->layout_cell = NULL;
+	}
+
+	w->saved_layout_root = w->layout_root;
+	layout_init(w, wp);
+	w->flags |= WINDOW_ZOOMED;
+
+	return (0);
+}
+
+int
+window_unzoom(struct window *w)
+{
+	struct window_pane	*wp;
+
+	if (!(w->flags & WINDOW_ZOOMED))
+		return (-1);
+
+	w->flags &= ~WINDOW_ZOOMED;
+	layout_free(w);
+	w->layout_root = w->saved_layout_root;
+
+	TAILQ_FOREACH(wp, &w->panes, entry) {
+		wp->layout_cell = wp->saved_layout_cell;
+		wp->saved_layout_cell = NULL;
+	}
+	layout_fix_panes(w, w->sx, w->sy);
+
+	return (0);
+}
+
 struct window_pane *
 window_add_pane(struct window *w, u_int hlimit)
 {
@@ -586,6 +641,8 @@ window_printable_flags(struct session *s, struct winlink *wl)
 		flags[pos++] = '*';
 	if (wl == TAILQ_FIRST(&s->lastw))
 		flags[pos++] = '-';
+	if (wl->window->flags & WINDOW_ZOOMED)
+		flags[pos++] = 'Z';
 	if (pos == 0)
 		flags[pos++] = ' ';
 	flags[pos] = '\0';
@@ -702,6 +759,8 @@ window_pane_spawn(struct window_pane *wp, const char *cmd, const char *shell,
 		wp->cwd = xstrdup(cwd);
 	}
 
+	log_debug("spawn: %s -- %s", wp->shell, wp->cmd);
+
 	memset(&ws, 0, sizeof ws);
 	ws.ws_col = screen_size_x(&wp->base);
 	ws.ws_row = screen_size_y(&wp->base);
@@ -804,7 +863,6 @@ window_pane_timer_callback(unused int fd, unused short events, void *data)
 	wp->changes = 0;
 }
 
-/* ARGSUSED */
 void
 window_pane_read_callback(unused struct bufferevent *bufev, void *data)
 {
@@ -831,7 +889,6 @@ window_pane_read_callback(unused struct bufferevent *bufev, void *data)
 		fatal("gettimeofday failed.");
 }
 
-/* ARGSUSED */
 void
 window_pane_error_callback(
     unused struct bufferevent *bufev, unused short what, void *data)
@@ -844,32 +901,16 @@ window_pane_error_callback(
 void
 window_pane_resize(struct window_pane *wp, u_int sx, u_int sy)
 {
-	struct winsize	ws;
-
 	if (sx == wp->sx && sy == wp->sy)
 		return;
 	wp->sx = sx;
 	wp->sy = sy;
 
-	memset(&ws, 0, sizeof ws);
-	ws.ws_col = sx;
-	ws.ws_row = sy;
-
-	screen_resize(&wp->base, sx, sy);
+	screen_resize(&wp->base, sx, sy, wp->saved_grid == NULL);
 	if (wp->mode != NULL)
 		wp->mode->resize(wp, sx, sy);
 
-	if (wp->fd != -1 && ioctl(wp->fd, TIOCSWINSZ, &ws) == -1)
-#ifdef __sun
-		/*
-		 * Some versions of Solaris apparently can return an error when
-		 * resizing; don't know why this happens, can't reproduce on
-		 * other platforms and ignoring it doesn't seem to cause any
-		 * issues.
-		 */
-		if (errno != EINVAL)
-#endif
-		fatal("ioctl failed");
+	wp->flags |= PANE_RESIZE;
 }
 
 /*
@@ -877,7 +918,8 @@ window_pane_resize(struct window_pane *wp, u_int sx, u_int sy)
  * history is not updated
  */
 void
-window_pane_alternate_on(struct window_pane *wp, struct grid_cell *gc)
+window_pane_alternate_on(struct window_pane *wp, struct grid_cell *gc,
+    int cursor)
 {
 	struct screen	*s = &wp->base;
 	u_int		 sx, sy;
@@ -891,8 +933,10 @@ window_pane_alternate_on(struct window_pane *wp, struct grid_cell *gc)
 
 	wp->saved_grid = grid_create(sx, sy, 0);
 	grid_duplicate_lines(wp->saved_grid, 0, s->grid, screen_hsize(s), sy);
-	wp->saved_cx = s->cx;
-	wp->saved_cy = s->cy;
+	if (cursor) {
+		wp->saved_cx = s->cx;
+		wp->saved_cy = s->cy;
+	}
 	memcpy(&wp->saved_cell, gc, sizeof wp->saved_cell);
 
 	grid_view_clear(s->grid, 0, 0, sx, sy);
@@ -904,7 +948,8 @@ window_pane_alternate_on(struct window_pane *wp, struct grid_cell *gc)
 
 /* Exit alternate screen mode and restore the copied grid. */
 void
-window_pane_alternate_off(struct window_pane *wp, struct grid_cell *gc)
+window_pane_alternate_off(struct window_pane *wp, struct grid_cell *gc,
+    int cursor)
 {
 	struct screen	*s = &wp->base;
 	u_int		 sx, sy;
@@ -921,14 +966,16 @@ window_pane_alternate_off(struct window_pane *wp, struct grid_cell *gc)
 	 * before copying back.
 	 */
 	if (sy > wp->saved_grid->sy)
-		screen_resize(s, sx, wp->saved_grid->sy);
+		screen_resize(s, sx, wp->saved_grid->sy, 1);
 
 	/* Restore the grid, cursor position and cell. */
 	grid_duplicate_lines(s->grid, screen_hsize(s), wp->saved_grid, 0, sy);
-	s->cx = wp->saved_cx;
+	if (cursor)
+		s->cx = wp->saved_cx;
 	if (s->cx > screen_size_x(s) - 1)
 		s->cx = screen_size_x(s) - 1;
-	s->cy = wp->saved_cy;
+	if (cursor)
+		s->cy = wp->saved_cy;
 	if (s->cy > screen_size_y(s) - 1)
 		s->cy = screen_size_y(s) - 1;
 	memcpy(gc, &wp->saved_cell, sizeof *gc);
@@ -938,8 +985,8 @@ window_pane_alternate_off(struct window_pane *wp, struct grid_cell *gc)
 	 * the current size.
 	 */
 	wp->base.grid->flags |= GRID_HISTORY;
-	if (sy > wp->saved_grid->sy)
-		screen_resize(s, sx, sy);
+	if (sy > wp->saved_grid->sy || sx != wp->saved_grid->sx)
+		screen_resize(s, sx, sy, 1);
 
 	grid_destroy(wp->saved_grid);
 	wp->saved_grid = NULL;
@@ -1021,7 +1068,7 @@ window_pane_mouse(
 		    options_get_number(&wp->window->options, "mode-mouse"))
 			wp->mode->mouse(wp, sess, m);
 	} else if (wp->fd != -1)
-		input_mouse(wp, m);
+		input_mouse(wp, sess, m);
 }
 
 int
@@ -1029,6 +1076,8 @@ window_pane_visible(struct window_pane *wp)
 {
 	struct window	*w = wp->window;
 
+	if (wp->layout_cell == NULL)
+		return (0);
 	if (wp->xoff >= w->sx || wp->yoff >= w->sy)
 		return (0);
 	if (wp->xoff + wp->sx > w->sx || wp->yoff + wp->sy > w->sy)
@@ -1203,7 +1252,7 @@ winlink_clear_flags(struct winlink *wl)
 void
 window_mode_attrs(struct grid_cell *gc, struct options *oo)
 {
-	memcpy(gc, &grid_default_cell, sizeof gc);
+	memcpy(gc, &grid_default_cell, sizeof *gc);
 	colour_set_fg(gc, options_get_number(oo, "mode-fg"));
 	colour_set_bg(gc, options_get_number(oo, "mode-bg"));
 	gc->attr |= options_get_number(oo, "mode-attr");
